package com.vendor.burn.model.cmdline.adapter;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.log.Log;
import com.vendor.burn.model.BurnContext;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.cmdline.ENandImg;
import com.vendor.burn.model.cmdline.adapter.CommonAdapter;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.devices.burners.CommonBurner;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.ip.NandIP500;
import com.vendor.burn.packagers.nand.ip.NandIP600;
import com.vendor.burn.packagers.nand.ip.NandIP610;
import com.vendor.burn.packagers.nand.ip.NandIP620;
import com.vendor.burn.packagers.nand.ip.NandIPCommon;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.views.FastbootView;
import java.util.ArrayList;
import org.eclipse.osgi.util.NLS;

public class NandImg extends CommonAdapter {
  private static final String[] BLOCK_SIZE_STRS = new String[] { "128K", "256K", "512K", "1M", "2M", "4M" };
  
  public static String[] getBlockSizeStrs() {
    return (String[])BLOCK_SIZE_STRS.clone();
  }
  
  private static final int[] BLOCK_SIZE_NUMS = new int[] { 131072, 262144, 524288, 1048576, 2097152, 4194304 };
  
  private static final int[] BLOCK_SIZE_SIZES_620 = new int[] { 131072, 262144, 524288, 1048576, 2097152, 4194304 };
  
  public static int[] getBlockSizeNums() {
    return (int[])BLOCK_SIZE_NUMS.clone();
  }
  
  public static int[] getBlockSizeSizes620() {
    return (int[])BLOCK_SIZE_SIZES_620.clone();
  }
  
  private static final String[] BLOCK_SIZE_STRS_620 = new String[] { "128K", "256K", "512K", "1M", "2M", "4M" };
  
  private UnknownDevice device;
  
  private String chipName;
  
  private String flash;
  
  private String inputFile;
  
  private boolean fastboot;
  
  private FlashOption flashOption;
  
  private FileSystem fileSystem;
  
  private boolean randomizer;
  
  private int block;
  
  private NandIPCommon nandIp;
  
  private CommonBurner burnBurner;
  
  public static String[] getBlockSizeStrs620() {
    return (String[])BLOCK_SIZE_STRS_620.clone();
  }
  
  private ArrayList<String> flashList = new ArrayList<>();
  
  private ArrayList<String> optionList;
  
  private ArrayList<String> blockList;
  
  private final String[] args;
  
  private final int paramSize;
  
  public NandImg(String[] args) {
    this.args = (String[])args.clone();
    this.paramSize = args.length - 1;
  }
  
  public boolean parseAllParameter() {
    if (!parseChipName()) {
      printNandImgEg();
      Log.println(Messages.NAND_IMG_GET_PAGESIZE_AND_BLOCK);
      return false;
    } 
    if (!parseFlash()) {
      printNandImgEg();
      Log.println(Messages.NAND_IMG_GET_PAGESIZE_AND_BLOCK);
      return false;
    } 
    if (!parseInputFile()) {
      printNandImg();
      return false;
    } 
    if (!parseFastboot()) {
      printNandImg();
      return false;
    } 
    if (!parsePageAndEcc()) {
      printNandImg();
      return false;
    } 
    if (!parseOobSize()) {
      printNandImg();
      return false;
    } 
    if (!parseFileSystem()) {
      printNandImg();
      return false;
    } 
    if (!parseRandomizer()) {
      printNandImg();
      return false;
    } 
    if (!parseBlock()) {
      printNandImg();
      return false;
    } 
    return true;
  }
  
  public boolean parseChipName() {
    if (this.paramSize >= ENandImg.chipName.ordinal()) {
      String chipName = this.args[ENandImg.chipName.ordinal()];
      boolean result = (chipName == null || chipName.isEmpty());
      if (result) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.chipName.name()));
        return false;
      } 
      String upperCaseChipName = convertChipName(chipName);
      UnknownDevice device = new UnknownDevice(upperCaseChipName);
      if (!device.isValid()) {
        Log.errorln(Messages.FastbootView_Unsupported_Device);
        return false;
      } 
      FastbootView.setCurrentChipNameType(device.getChipNameType());
      if (checkNandSupport(device, device.getNandIPVer()) && checkNandSupport(device, device.getSpinandIPVer())) {
        Log.errorln(Messages.NandImage_DoNotSupport);
        return false;
      } 
      if (device.getNandIPVer() != -1)
        this.flashList.add(EFlash.nand.name()); 
      if (device.getSpinandIPVer() != -1)
        this.flashList.add(EFlash.spinand.name()); 
      setChipName(upperCaseChipName);
      setDevice(device);
      return true;
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.chipName.name()));
    return false;
  }
  
  public boolean checkNandSupport(UnknownDevice device, int nandIPVer) {
    boolean isSupportNand = (nandIPVer != 500 && nandIPVer != 600 && nandIPVer != 610 && nandIPVer != 611 && nandIPVer != 620);
    return isSupportNand;
  }
  
  public boolean parseFlash() {
    if (this.paramSize >= ENandImg.flash.ordinal()) {
      String flashStr = this.args[ENandImg.flash.ordinal()];
      if (flashStr == null || !checkFlash(flashStr))
        return false; 
      NandIPCommon nandIp = null;
      int nandIPVer = this.device.getNandIPVer();
      int spinandIPVer = this.device.getSpinandIPVer();
      boolean isAllSupport = (nandIPVer != -1 && spinandIPVer != -1);
      if (isAllSupport) {
        if (flashStr.equalsIgnoreCase("Nand")) {
          nandIp = chooseNandVersion("Nand", nandIPVer);
        } else if (flashStr.equalsIgnoreCase("SPINand")) {
          nandIp = chooseNandVersion("SPINand", spinandIPVer);
        } 
      } else if (nandIPVer != -1) {
        nandIp = chooseNandVersion("Nand", nandIPVer);
      } else if (spinandIPVer != -1) {
        nandIp = chooseNandVersion("SPINand", spinandIPVer);
      } else {
        nandIp = chooseNandVersion("Nand", nandIPVer);
      } 
      setFlash(flashStr);
      setNandIp(nandIp);
      BurnContext burnContext = this.device.getBurnContext();
      if (burnContext != null && nandIp != null) {
        this.burnBurner = new CommonBurner(burnContext);
        this.optionList = getPageSizeEccType(this.burnBurner, nandIp);
        this.blockList = getblockLists(nandIp);
        return true;
      } 
      Log.errorln("BurnContext is null!");
      return false;
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.flash.name()));
    printFlashOptions(this.flashList);
    return false;
  }
  
  public boolean parseInputFile() {
    if (this.paramSize >= ENandImg.inputFile.ordinal()) {
      String inputFile = this.args[ENandImg.inputFile.ordinal()];
      if (inputFile == null || inputFile.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.inputFile.name()));
        return false;
      } 
      if (!FileUtils.isExists(inputFile)) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_FilePath_IsNotExist, inputFile));
        return false;
      } 
      setInputFile(inputFile);
    } else {
      Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.inputFile.name()));
      return false;
    } 
    return true;
  }
  
  public boolean parseFastboot() {
    if (this.paramSize >= ENandImg.fastboot.ordinal()) {
      String fastbootStr = this.args[ENandImg.fastboot.ordinal()];
      if (fastbootStr == null || fastbootStr.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.fastboot.name()));
        printFastbootOptions();
        return false;
      } 
      try {
        int fastboot = Integer.parseInt(fastbootStr);
        if (fastboot == EFastboot.un_fastboot.ordinal()) {
          setFastboot(false);
        } else if (fastboot == EFastboot.fastboot.ordinal()) {
          setFastboot(true);
        } else {
          Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.fastboot.name()));
          printFastbootOptions();
          return false;
        } 
        return true;
      } catch (NumberFormatException e) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.fastboot.name()));
        printFastbootOptions();
        return false;
      } 
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.fastboot.name()));
    printFastbootOptions();
    return false;
  }
  
  public boolean parsePageAndEcc() {
    if (this.nandIp == null) {
      Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.flash.name()));
      return false;
    } 
    if (this.paramSize >= ENandImg.pageSizeEccType.ordinal()) {
      String pageSizeEccTypeStr = this.args[ENandImg.pageSizeEccType.ordinal()];
      if (pageSizeEccTypeStr == null || pageSizeEccTypeStr.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.pageSizeEccType.name()));
        printPageEccOptions(this.optionList);
        return false;
      } 
      int pageSizeEccType = 0;
      try {
        pageSizeEccType = Integer.parseInt(pageSizeEccTypeStr);
      } catch (NumberFormatException e) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.pageSizeEccType.name()));
        return false;
      } 
      if (!checkPageSizeEccType(this.optionList, pageSizeEccType)) {
        printPageEccOptions(this.optionList);
        return false;
      } 
      FlashOption[] options = this.nandIp.getAvailableOptions((IBurner)this.burnBurner);
      if (options != null) {
        FlashOption flashOption = options[pageSizeEccType];
        setFlashOption(flashOption);
        setOptionList(this.optionList);
        return true;
      } 
      Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IS_NULL, "FlashOption"));
      return false;
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.pageSizeEccType.name()));
    printPageEccOptions(this.optionList);
    return false;
  }
  
  public boolean parseOobSize() {
    if (this.paramSize >= ENandImg.oobsize.ordinal()) {
      String oobsizeStr = this.args[ENandImg.oobsize.ordinal()];
      if (oobsizeStr == null || oobsizeStr.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.oobsize.name()));
        return false;
      } 
      try {
        int oobSize = Integer.parseInt(oobsizeStr);
        FlashOption flashOption = getFlashOption();
        flashOption.setOOBSize(oobSize);
        return true;
      } catch (NumberFormatException e) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.oobsize.name()));
        return false;
      } 
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.oobsize.name()));
    return false;
  }
  
  public boolean parseFileSystem() {
    if (this.paramSize >= ENandImg.fileSystem.ordinal()) {
      String fileSystemStr = this.args[ENandImg.fileSystem.ordinal()];
      if (fileSystemStr == null || fileSystemStr.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.fileSystem.name()));
        printFileSystemOptions();
        return false;
      } 
      try {
        int fileSystem = Integer.parseInt(fileSystemStr);
        if (fileSystem == EFileSystem.none.ordinal()) {
          setFileSystem(FileSystem.none);
        } else if (fileSystem == EFileSystem.yaffs.ordinal()) {
          setFileSystem(FileSystem.yaffs);
        } else if (fileSystem == EFileSystem.ubiFs.ordinal()) {
          setFileSystem(FileSystem.ubiFs);
        } else {
          Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.fileSystem.name()));
          printFileSystemOptions();
          return false;
        } 
        return true;
      } catch (NumberFormatException e) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.fileSystem.name()));
        printFileSystemOptions();
        return false;
      } 
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.fileSystem.name()));
    printFileSystemOptions();
    return false;
  }
  
  public boolean parseRandomizer() {
    if (this.paramSize >= ENandImg.randomizer.ordinal()) {
      String randomizerStr = this.args[ENandImg.randomizer.ordinal()];
      if (randomizerStr == null || randomizerStr.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.randomizer.name()));
        printRandomizerOptions();
        return false;
      } 
      try {
        int randomizer = Integer.parseInt(randomizerStr);
        if (randomizer == ERandomizer.randomizer_disabled.ordinal()) {
          setRandomizer(false);
        } else if (randomizer == ERandomizer.randomizer_enabled.ordinal()) {
          setRandomizer(true);
        } else {
          Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.randomizer.name()));
          printRandomizerOptions();
          return false;
        } 
        return true;
      } catch (NumberFormatException e) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.randomizer.name()));
        printRandomizerOptions();
        return false;
      } 
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.randomizer.name()));
    printRandomizerOptions();
    return false;
  }
  
  public boolean parseBlock() {
    if (this.nandIp == null) {
      Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.flash.name()));
      return false;
    } 
    if (this.paramSize >= ENandImg.block.ordinal()) {
      String blockStr = this.args[ENandImg.block.ordinal()];
      if (blockStr == null || blockStr.isEmpty()) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.block.name()));
        printBlockOptions(this.blockList);
        return false;
      } 
      try {
        int block = Integer.parseInt(blockStr);
        if (block > this.blockList.size() - 1) {
          Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.block.name()));
          printBlockOptions(this.blockList);
          return false;
        } 
        setBlock(BLOCK_SIZE_NUMS[block]);
        setBlockList(this.blockList);
      } catch (NumberFormatException e) {
        Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.block.name()));
        printBlockOptions(this.blockList);
        return false;
      } 
      return true;
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_NotFound, ENandImg.block.name()));
    printBlockOptions(this.blockList);
    return false;
  }
  
  private NandIPCommon chooseNandVersion(String typeIp, int nandIPVer) {
    NandIP620 nandIP620;
    NandIPCommon nandSet = null;
    if (nandIPVer == 500) {
      NandIP500 nandIP500 = new NandIP500();
    } else if (nandIPVer == 600) {
      NandIP600 nandIP600 = new NandIP600();
    } else if (nandIPVer == 610) {
      NandIP610 nandIP610 = new NandIP610();
    } else if (nandIPVer == 620) {
      nandIP620 = new NandIP620();
    } else {
      Log.errorln(Messages.NandImage_DoNotSupport);
    } 
    if (nandIP620 != null)
      nandIP620.setCurSelectedNandWay(typeIp); 
    return (NandIPCommon)nandIP620;
  }
  
  private ArrayList<String> getPageSizeEccType(CommonBurner burnBurner, NandIPCommon nandIp) {
    ArrayList<String> optionList = new ArrayList<>();
    FlashOption[] options = nandIp.getAvailableOptions((IBurner)burnBurner);
    if (options != null)
      for (FlashOption flashOption : options)
        optionList.add(flashOption.toString());  
    return optionList;
  }
  
  private ArrayList<String> getblockLists(NandIPCommon nandIp) {
    ArrayList<String> blockList = new ArrayList<>();
    if (nandIp == null)
      return blockList; 
    for (int i = 0; i < BLOCK_SIZE_STRS.length; i++)
      blockList.add(BLOCK_SIZE_STRS[i]); 
    return blockList;
  }
  
  private boolean checkPageSizeEccType(ArrayList<String> optionLists, int pageSizeEccType) {
    if (optionLists == null)
      return false; 
    if (pageSizeEccType > optionLists.size() - 1) {
      Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.pageSizeEccType.name()));
      return false;
    } 
    return true;
  }
  
  private boolean checkFlash(String flashStr) {
    boolean result = (flashStr == null || flashStr.isEmpty());
    if (result) {
      Log.errorln(NLS.bind(Messages.CMD_PARAMETER_IsNull, ENandImg.flash.name()));
      printFlashOptions(this.flashList);
      return false;
    } 
    EFlash[] eFlashs = EFlash.values();
    for (EFlash eFlash : eFlashs) {
      if (eFlash.name().equalsIgnoreCase(flashStr))
        return true; 
    } 
    Log.errorln(NLS.bind(Messages.CMD_PARAMETER_Invalid, ENandImg.flash.name()));
    printFlashOptions(this.flashList);
    return false;
  }
  
  public void printNandImg() {
    printNandImgEg();
    printFlashOptions(this.flashList);
    printInputFileOption();
    printFastbootOptions();
    printPageEccOptions(this.optionList);
    printOobSizeOption();
    printFileSystemOptions();
    printRandomizerOptions();
    printBlockOptions(this.blockList);
  }
  
  public static void printNandImgEg() {
    Log.println("burntoolcli.jar --nandimg  <chipName>  <flash> <inputFile>  <fastboot> <pagesize/ecctype> <oobsize> <fileSystem> <randomizer> <block>");
    Log.println("burntoolcli.jar --nandimg  chipName   nand  In_2k4b.yaffs     0              0             64          1            0         0   ");
  }
  
  private void printFlashOptions(ArrayList<String> flashList) {
    Log.println("Flash:");
    if (flashList != null && flashList.size() > 0) {
      for (String flash : flashList)
        Log.println(flash); 
    } else {
      Log.println(EFlash.nand.name());
      Log.errorln(Messages.NAND_IMG_GET_FLASH);
    } 
  }
  
  private void printInputFileOption() {
    Log.println("Input file:");
  }
  
  private void printPageEccOptions(ArrayList<String> optionList) {
    Log.println("PageSize/EccType:");
    if (optionList != null && optionList.size() > 0) {
      for (int i = 0; i < optionList.size(); i++) {
        String option = optionList.get(i);
        if (option != null)
          Log.println("" + i + "    " + i); 
      } 
    } else {
      Log.errorln(Messages.NAND_IMG_GET_PAGESIZE_AND_BLOCK);
    } 
  }
  
  private void printOobSizeOption() {
    Log.println("Chip OOB size:");
  }
  
  private void printFileSystemOptions() {
    Log.println("FileSystem:");
    EFileSystem[] eFileSystems = EFileSystem.values();
    for (EFileSystem eFileSystem : eFileSystems)
      Log.println("" + eFileSystem.ordinal() + "    " + eFileSystem.ordinal()); 
  }
  
  private void printRandomizerOptions() {
    Log.println("Randomizer:");
    ERandomizer[] eRandomizers = ERandomizer.values();
    for (ERandomizer eRandomizer : eRandomizers)
      Log.println("" + eRandomizer.ordinal() + "    " + eRandomizer.ordinal()); 
  }
  
  private void printFastbootOptions() {
    Log.println("Fastboot:");
    EFastboot[] efastboots = EFastboot.values();
    for (EFastboot eFastboot : efastboots)
      Log.println("" + eFastboot.ordinal() + "    " + eFastboot.ordinal()); 
  }
  
  private void printBlockOptions(ArrayList<String> blockLists) {
    Log.println("block:");
    if (blockLists != null && blockLists.size() > 0) {
      for (int i = 0; i < blockLists.size(); i++) {
        String block = blockLists.get(i);
        if (block != null)
          Log.println("" + i + "    " + i); 
      } 
    } else {
      Log.errorln(Messages.NAND_IMG_GET_PAGESIZE_AND_BLOCK);
    } 
  }
  
  public String getChipName() {
    return this.chipName;
  }
  
  public void setChipName(String chipName) {
    this.chipName = chipName;
  }
  
  public String getFlash() {
    return this.flash;
  }
  
  public void setFlash(String flash) {
    this.flash = flash;
  }
  
  public String getInputFile() {
    return this.inputFile;
  }
  
  public void setInputFile(String inputFile) {
    this.inputFile = inputFile;
  }
  
  public FlashOption getFlashOption() {
    return this.flashOption;
  }
  
  public void setFlashOption(FlashOption flashOption) {
    this.flashOption = flashOption;
  }
  
  public FileSystem getFileSystem() {
    return this.fileSystem;
  }
  
  public void setFileSystem(FileSystem fileSystem) {
    this.fileSystem = fileSystem;
  }
  
  public boolean isRandomizer() {
    return this.randomizer;
  }
  
  public void setRandomizer(boolean randomizer) {
    this.randomizer = randomizer;
  }
  
  public NandIPCommon getNandIp() {
    return this.nandIp;
  }
  
  public void setNandIp(NandIPCommon nandIp) {
    this.nandIp = nandIp;
  }
  
  public int getBlock() {
    return this.block;
  }
  
  public void setBlock(int block) {
    this.block = block;
  }
  
  public ArrayList<String> getOptionList() {
    return this.optionList;
  }
  
  public void setOptionList(ArrayList<String> optionList) {
    this.optionList = optionList;
  }
  
  public ArrayList<String> getBlockList() {
    return this.blockList;
  }
  
  public void setBlockList(ArrayList<String> blockList) {
    this.blockList = blockList;
  }
  
  public UnknownDevice getDevice() {
    return this.device;
  }
  
  public void setDevice(UnknownDevice device) {
    this.device = device;
  }
  
  public ArrayList<String> getFlashList() {
    return this.flashList;
  }
  
  public void setFlashList(ArrayList<String> flashList) {
    this.flashList = flashList;
  }
  
  public boolean isFastboot() {
    return this.fastboot;
  }
  
  public void setFastboot(boolean fastboot) {
    this.fastboot = fastboot;
  }
}
